import importlib
import os
from typing import Optional

from langchain_community.llms.cohere import Cohere

from embedchain_crewai.config import BaseLlmConfig
from embedchain_crewai.helpers.json_serializable import register_deserializable
from embedchain_crewai.llm.base import BaseLlm


@register_deserializable
class CohereLlm(BaseLlm):
    def __init__(self, config: Optional[BaseLlmConfig] = None):
        try:
            importlib.import_module("cohere")
        except ModuleNotFoundError:
            raise ModuleNotFoundError(
                "The required dependencies for Cohere are not installed."
                'Please install with `pip install --upgrade "embedchain[cohere]"`'
            ) from None

        super().__init__(config=config)
        if not self.config.api_key and "COHERE_API_KEY" not in os.environ:
            raise ValueError("Please set the COHERE_API_KEY environment variable or pass it in the config.")

    def get_llm_model_answer(self, prompt):
        if self.config.system_prompt:
            raise ValueError("CohereLlm does not support `system_prompt`")
        return CohereLlm._get_answer(prompt=prompt, config=self.config)

    @staticmethod
    def _get_answer(prompt: str, config: BaseLlmConfig) -> str:
        api_key = config.api_key or os.getenv("COHERE_API_KEY")
        llm = Cohere(
            cohere_api_key=api_key,
            model=config.model,
            max_tokens=config.max_tokens,
            temperature=config.temperature,
            p=config.top_p,
        )

        return llm.invoke(prompt)
